﻿#include <iostream>
#include <unistd.h>
#include "../Include/FAS_EziMOTIONPlusE.h"

using namespace PE;

bool Connect(int nCommType, int nBdID);
bool CheckDriveErr(int nBdID);
bool SetServoOn(int nBdID);
bool PosIncOverride(int nBdID);
bool PosAbsOvrride(int nBdID);

#define TCP 0
#define UDP 1

#define INCPOS 200000 // IncMove Target Position
#define ABSPOS 0 // AbsMove Target Position

int main()
{
	int nBdID = 0;

	// Device Connect
	if (Connect(TCP, nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Drive Error Check
	if (CheckDriveErr(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// ServoOn 
	if (SetServoOn(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Move AxisIncPos & PositionIncOverride
	if (PosIncOverride(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Move AxisAbsPos & PositionAbsOverride
	if (PosAbsOvrride(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Connection Close
	FAS_Close(nBdID);

	getchar();
}

bool Connect(int nCommType, int nBdID)
{
	unsigned char byIP[4] = { 192, 168, 0, 2 }; // IP : 192.168.0.2
	bool bSuccess = true;

	// Connection
	switch (nCommType)
	{
	case TCP:
		// TCP Connection
		if (FAS_ConnectTCP(byIP[0], byIP[1], byIP[2], byIP[3], nBdID) == 0)
		{
			printf("TCP Connection Fail! \n");
			bSuccess = false;
		}
		break;

	case UDP:
		// UDP Connection
		if (FAS_Connect(byIP[0], byIP[1], byIP[2], byIP[3], nBdID) == 0)
		{
			printf("UDP Connection Fail! \n");
			bSuccess = false;
		}
		break;

	default:
		printf("Wrong communication type. \n");
		bSuccess = false;

		break;
	}

	if (bSuccess)
		printf("Connected successfully. \n");

	return bSuccess;
}

bool CheckDriveErr(int nBdID)
{
	// Check Drive's Error
	EZISERVO2_AXISSTATUS AxisStatus;

	if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
	{
		printf("Function(FAS_GetAxisStatus) was failed.\n");
		return false;
	}

	if (AxisStatus.FFLAG_ERRORALL)
	{
		// if Drive's Error was detected, Reset the ServoAlarm
		if (FAS_ServoAlarmReset(nBdID) != FMM_OK)
		{
			printf("Function(FAS_ServoAlarmReset) was failed.\n");
			return false;
		}
	}

	return true;
}

bool SetServoOn(int nBdID)
{
	// Check Drive's Servo Status
	EZISERVO2_AXISSTATUS AxisStatus;

	// if ServoOnFlagBit is OFF('0'), switch to ON('1')
	if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
	{
		printf("Function(FAS_GetAxisStatus) was failed.\n");
		return false;
	}

	if (AxisStatus.FFLAG_SERVOON == 0)
	{
		if (FAS_ServoEnable(nBdID, 1) != FMM_OK)
		{
			printf("Function(FAS_ServoEnable) was failed.\n");
			return false;
		}

		do
		{
			usleep(1000);

			if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
			{
				printf("Function(FAS_GetAxisStatus) was failed.\n");
				return false;
			}

			if (AxisStatus.FFLAG_SERVOON)
				printf("Servo ON \n");
		}
		while (!AxisStatus.FFLAG_SERVOON); // Wait until FFLAG_SERVOON is ON
	}
	else
	{
		printf("Servo is already ON \n");
	}

	return true;
}

bool PosIncOverride(int nBdID)
{
	// Move AxisIncPos & PositionIncOverride
	EZISERVO2_AXISSTATUS AxisStatus;
	int lIncPos = INCPOS;
	int lVelocity = 20000;
	int lActualPos = 0;
	int lChangePos = 0;
	int lPosDetect = 100000;

	printf("---------------------------\n");
	printf("[Inc Mode] Move Motor Start! \n");

	// 1. Move Command
	// Move the motor by INCPOS(200000) [pulse]
	if (FAS_MoveSingleAxisIncPos(nBdID, lIncPos, lVelocity) != FMM_OK)
	{
		printf("Function(FAS_MoveSingleAxisAbsPos) was failed.\n");
		return false;
	}

	// 2. Check Condition
	// Check current position
	do
	{
		usleep(1000);

		if (FAS_GetActualPos(nBdID, &lActualPos) != FMM_OK)
		{
			printf("Function(FAS_GetActualPos) was failed.\n");
			return false;
		}
	}
	while (lActualPos <= (lPosDetect));

	// 3. Change Position
	// If the current position is less than the target position, change final position.
	lChangePos += lIncPos;
	if (FAS_PositionIncOverride(nBdID, lChangePos) != FMM_OK)
	{
		printf("Function(FAS_PositionIncOverride) was failed.\n");
		return false;
	}
	else
		printf("[Before] Target Position : %d[pulse] / [After] Target Position : %d[pulse] \n", lIncPos, lChangePos + lIncPos);

	// 4. Confirm Move Complete
	// Check the Axis status until motor stops and the Inposition value is checked
	do
	{
		usleep(1000);

		if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
		{
			printf("Function(FAS_GetAxisStatus) was failed.\n");
			return false;
		}
	}
	while (AxisStatus.FFLAG_MOTIONING || !(AxisStatus.FFLAG_INPOSITION));

	return true;
}

bool PosAbsOvrride(int nBdID)
{
	// Move AxisAbsPos & PositionAbsOverride
	EZISERVO2_AXISSTATUS AxisStatus;
	int lVelocity = 40000;
	int lIncEndPos = 0;
	int lActualPos = 0;
	int lAbsPos;
	int lChangePos = 0;

	if (FAS_GetActualPos(nBdID, &lIncEndPos) != FMM_OK)
	{
		printf("Function(FAS_GetActualPos) was failed.\n");
		return false;
	}

	printf("---------------------------\n");

	// 1. Move Command
	// Move the motor by ((lIncEndPos)* 1/ 4) pulse (target position : Absolute position)
	lAbsPos = lIncEndPos / 4;

	if (FAS_MoveSingleAxisAbsPos(nBdID, lAbsPos, lVelocity) != FMM_OK)
	{
		printf("Function(FAS_MoveSingleAxisAbsPos) was failed.\n");
		return false;
	}
	printf("[ABS Mode] Move Motor Start! \n");

	// 2. Check Condition
	// Check current position
	do
	{
		usleep(1000);

		if (FAS_GetActualPos(nBdID, &lActualPos) != FMM_OK)
		{
			printf("Function(FAS_GetActualPos) was failed.\n");
			return false;
		}
	}
	while (lActualPos >= (lIncEndPos / 2));

	// 3. Change Position
	// if the current position falls below half the INC End position, change the target position to zero.
	lChangePos = ABSPOS;
	if (FAS_PositionAbsOverride(nBdID, lChangePos) != FMM_OK)
	{
		printf("Function(FAS_PositionAbsOverride) was failed.\n");
		return false;
	}
	else
		printf("Before Target Posistion : %d[pulse] / Change Target Posistion : %d[pulse] \n", lAbsPos, lChangePos);

	// 4. Confirm Move Complete
	// Check the Axis status until motor stops and the Inposition value is checked
	do
	{
		usleep(1000);

		if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
		{
			printf("Function(FAS_GetAxisStatus) was failed.\n");
			return false;
		}

	}
	while (AxisStatus.FFLAG_MOTIONING || !(AxisStatus.FFLAG_INPOSITION));

	return true;
}
