﻿#include <iostream>
#include <unistd.h>
#include "../Include/FAS_EziMOTIONPlusE.h"

using namespace PE;

bool Connect(int nCommType, int nBdID);
bool CheckDriveErr(int nBdID);
bool SetServoOn(int nBdID);
bool MovePosAndVelOverride(int nBdID);
bool JogAndVelOverride(int nBdID);

#define TCP 0
#define UDP 1

#define CW  1
#define CCW 0

#define INCPOS 200000 // IncMove Target Position

int main()
{
	int nBdID = 0;

	// Device Connect
	if (Connect(TCP, nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Drive Error Check
	if (CheckDriveErr(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// ServoOn 
	if (SetServoOn(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Move AxisIncPos & VelocityOverride
	if (MovePosAndVelOverride(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// MoveVelocity  & VelocityOverride
	if (JogAndVelOverride(nBdID) == false)
	{
		getchar();
		exit(1);
	}

	// Connection Close
	FAS_Close(nBdID);

	getchar();
}

bool Connect(int nCommType, int nBdID)
{
	unsigned char byIP[4] = { 192, 168, 0, 2 }; // IP : 192.168.0.2
	bool bSuccess = true;

	// Connection
	switch (nCommType)
	{
	case TCP:
		// TCP Connection
		if (FAS_ConnectTCP(byIP[0], byIP[1], byIP[2], byIP[3], nBdID) == 0)
		{
			printf("TCP Connection Fail! \n");
			bSuccess = false;
		}
		break;

	case UDP:
		// UDP Connection
		if (FAS_Connect(byIP[0], byIP[1], byIP[2], byIP[3], nBdID) == 0)
		{
			printf("UDP Connection Fail! \n");
			bSuccess = false;
		}
		break;

	default:
		printf("Wrong communication type. \n");
		bSuccess = false;

		break;
	}

	if (bSuccess)
		printf("Connected successfully. \n");

	return bSuccess;
}

bool CheckDriveErr(int nBdID)
{
	// Check Drive's Error
	EZISERVO2_AXISSTATUS AxisStatus;

	if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
	{
		printf("Function(FAS_GetAxisStatus) was failed.\n");
		return false;
	}

	if (AxisStatus.FFLAG_ERRORALL)
	{
		// if Drive's Error was detected, Reset the ServoAlarm
		if (FAS_ServoAlarmReset(nBdID) != FMM_OK)
		{
			printf("Function(FAS_ServoAlarmReset) was failed.\n");
			return false;
		}
	}

	return true;
}

bool SetServoOn(int nBdID)
{
	// Check Drive's Servo Status
	EZISERVO2_AXISSTATUS AxisStatus;

	// if ServoOnFlagBit is OFF('0'), switch to ON('1')
	if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
	{
		printf("Function(FAS_GetAxisStatus) was failed.\n");
		return false;
	}

	if (AxisStatus.FFLAG_SERVOON == 0)
	{
		if (FAS_ServoEnable(nBdID, 1) != FMM_OK)
		{
			printf("Function(FAS_ServoEnable) was failed.\n");
			return false;
		}

		do
		{
			usleep(1000);

			if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
			{
				printf("Function(FAS_GetAxisStatus) was failed.\n");
				return false;
			}

			if (AxisStatus.FFLAG_SERVOON)
				printf("Servo ON \n");
		}
		while (!AxisStatus.FFLAG_SERVOON); // Wait until FFLAG_SERVOON is ON
	}
	else
	{
		printf("Servo is already ON \n");
	}

	return true;
}

bool MovePosAndVelOverride(int nBdID)
{
	// MoveSingleAxisIncPos & VelocityOverride

	// Move the motor by INCPOS(200000) [pulse]
	EZISERVO2_AXISSTATUS AxisStatus;
	int lVelocity = 20000;
	int lActualPos = 0;
	int lPosDetect = 100000;
	int lIncPos = INCPOS;
	int lChangeVelocity = 0;

	printf("---------------------------\n");
	printf("[Inc Mode] Move Motor Start! \n");

	// 1. Move Command
	// Move the motor by INCPOS(200000) [pulse]
	if (FAS_MoveSingleAxisIncPos(nBdID, lIncPos, lVelocity) != FMM_OK)
	{
		printf("Function(FAS_MoveSingleAxisAbsPos) was failed.\n");
		return false;
	}

	// 2. Check Condition
	// Check current position
	do
	{
		usleep(1000);

		if (FAS_GetActualPos(nBdID, &lActualPos) != FMM_OK)
		{
			printf("Function(FAS_GetActualPos) was failed.\n");
			return false;
		}
	}
	while (lActualPos <= (lPosDetect));

	// 3. Change Velocity
	// If the current position is less than the target position, change velocity.
	lChangeVelocity = lVelocity * 2;
	if (FAS_VelocityOverride(nBdID, lChangeVelocity) != FMM_OK)
	{
		printf("Function(FAS_VelocityOverride) was failed.\n");
		return false;
	}
	else
		printf("Before Velocity : %d[pps] / Change Velocity : %d[pps] \n", lVelocity, lChangeVelocity);

	// 4. Confirm Move Complete
	// Check the Axis status until motor stops and the Inposition value is checked
	do
	{
		usleep(1000);

		if (FAS_GetAxisStatus(nBdID, &(AxisStatus.dwValue)) != FMM_OK)
		{
			printf("Function(FAS_GetAxisStatus) was failed.\n");
			return false;
		}

	}
	while (AxisStatus.FFLAG_MOTIONING || !(AxisStatus.FFLAG_INPOSITION));

	return true;
}

bool JogAndVelOverride(int nBdID)
{
	// MoveVelocity & VelocityOverride

	int lVelocity = 10000;
	int lChangeVelocity = 0;
	int lActualVelocity = 0;

	printf("---------------------------\n");

	// 1. Move Command
	// Move the motor 
	if (FAS_MoveVelocity(nBdID, lVelocity, CW) != FMM_OK)
	{
		printf("Function(FAS_MoveVelocity) was failed. \n");
		return false;
	}
	else
	{
		printf("Current Velocity : %d \n", lVelocity);
	}

	printf(">> Wait... 3 Second \n");
	usleep(3 * 1000 * 1000);

	// 2. Change Velocity
	if (FAS_GetActualVel(nBdID, &lActualVelocity) != FMM_OK)
	{
		printf("Function(FAS_GetActualVel) was failed. \n");
		return false;
	}

	lChangeVelocity = lActualVelocity * 2;
	if (FAS_VelocityOverride(nBdID, lChangeVelocity) != FMM_OK)
	{
		printf("Function(FAS_VelocityOverride) was failed. \n");
		return false;
	}
	else
		printf("Before Velocity : %d [pps] / Change Velocity : %d [pps] \n", lActualVelocity, lChangeVelocity);

	printf(">> Wait... 5 Second \n");
	usleep(5 * 1000 * 1000);

	// 3. Change Velocity
	if (FAS_GetActualVel(nBdID, &lActualVelocity) != FMM_OK)
	{
		printf("Function(FAS_GetActualVel) was failed. \n");
		return false;
	}

	lChangeVelocity = lActualVelocity * 2;
	if (FAS_VelocityOverride(nBdID, lChangeVelocity) != FMM_OK)
	{
		printf("Function(FAS_VelocityOverride) was failed. \n");
		return false;
	}
	else
		printf("Before Velocity : %d [pps] / Change Velocity : %d [pps] \n", lActualVelocity, lChangeVelocity);

	printf(">> Wait... 10 Second \n");
	usleep(10 * 1000 * 1000);

	// 4. Stop
	FAS_MoveStop(nBdID);
	printf("FAS_MoveStop! \n");

	return true;
}
